﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using System.Linq;
using NHibernate.Dialect;
using NUnit.Framework;
using NHibernate.Linq;
using System.Linq.Expressions;
using NHibernate.Linq.Functions;
using NHibernate.Hql.Ast;
using System.Reflection;
using System.Collections.ObjectModel;
using NHibernate.Linq.Visitors;
using NHibernate.Cfg;

namespace NHibernate.Test.NHSpecificTest.NH2869
{
    public static class MyLinqExtensions
    {


        public static int IsOneInDbZeroInLocal(this string source, string other)
        {
            return 0;
        }
    }

    public class MyLinqToHqlGeneratorsRegistry : DefaultLinqToHqlGeneratorsRegistry
    {
        public MyLinqToHqlGeneratorsRegistry()
            : base()
        {
            RegisterGenerator(ReflectionHelper.GetMethodDefinition(() => MyLinqExtensions.IsOneInDbZeroInLocal(null,null)),
                              new IsTrueInDbFalseInLocalGenerator());
        }
    }



    public class IsTrueInDbFalseInLocalGenerator : BaseHqlGeneratorForMethod
    {
        public IsTrueInDbFalseInLocalGenerator()
        {
            SupportedMethods = new[] { ReflectionHelper.GetMethodDefinition(() => MyLinqExtensions.IsOneInDbZeroInLocal(null,null)) };
        }

        public override HqlTreeNode BuildHql(MethodInfo method, Expression targetObject,
            ReadOnlyCollection<Expression> arguments, HqlTreeBuilder treeBuilder, IHqlExpressionVisitor visitor)
        {
            return treeBuilder.Constant(1);
        }
    }

    [TestFixture]
    public class Fixture : BugTestCase
    {

        protected override void Configure(Cfg.Configuration configuration)
        {
            configuration.LinqToHqlGeneratorsRegistry<MyLinqToHqlGeneratorsRegistry>();
            base.Configure(configuration);
        }
        protected override void OnSetUp()
        {
            base.OnSetUp();
            using (ISession session = this.OpenSession())
            {
                DomainClass entity = new DomainClass();
                entity.Id = 1;
                entity.Name = "Test";
                session.Save(entity);
                session.Flush();
            }
        }

        protected override void OnTearDown()
        {
            base.OnTearDown();
            using (ISession session = this.OpenSession())
            {
                string hql = "from System.Object";
                session.Delete(hql);
                session.Flush();
            }
        }

        protected override bool AppliesTo(NHibernate.Dialect.Dialect dialect)
        {
            return dialect as MsSql2005Dialect != null;
        }

        
        [Test]
        public void CustomExtensionWithConstantArgumentShouldBeIncludedInHqlProjection()
        {
            using (ISession session = this.OpenSession())
            {
                var projectionValue = (from c in session.Query<DomainClass>() where c.Name.IsOneInDbZeroInLocal("test") == 1 select c.Name.IsOneInDbZeroInLocal("test")).FirstOrDefault();
                //If the value is 0, the call was done in .NET, if it's 1 it has been projected correctly
                Assert.AreEqual(1, projectionValue);
            }
        }
    }
}
